/*******************************************************************************
* File Name: SER_SPI_UART.c
* Version 3.0
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  SPI and UART modes.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SER_PVT.h"
#include "SER_SPI_UART_PVT.h"

/***************************************
*        SPI/UART Private Vars
***************************************/

#if(SER_INTERNAL_RX_SW_BUFFER_CONST)
    volatile uint32 SER_rxBufferHead;
    volatile uint32 SER_rxBufferTail;
    volatile uint8  SER_rxBufferOverflow;
#endif /* (SER_INTERNAL_RX_SW_BUFFER_CONST) */

#if(SER_INTERNAL_TX_SW_BUFFER_CONST)
    volatile uint32 SER_txBufferHead;
    volatile uint32 SER_txBufferTail;
#endif /* (SER_INTERNAL_TX_SW_BUFFER_CONST) */

#if(SER_INTERNAL_RX_SW_BUFFER)
    /* Add one element to the buffer to receive full packet. One byte in receive buffer is always empty */
    volatile uint8 SER_rxBufferInternal[SER_INTERNAL_RX_BUFFER_SIZE];
#endif /* (SER_INTERNAL_RX_SW_BUFFER) */

#if(SER_INTERNAL_TX_SW_BUFFER)
    volatile uint8 SER_txBufferInternal[SER_TX_BUFFER_SIZE];
#endif /* (SER_INTERNAL_TX_SW_BUFFER) */


#if(SER_RX_DIRECTION)
    /*******************************************************************************
    * Function Name: SER_SpiUartReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Retrieves the next data element from the receive buffer.
    *   - RX software buffer is disabled: Returns data element retrieved from
    *     RX FIFO. Undefined data will be returned if the RX FIFO is empty.
    *   - RX software buffer is enabled: Returns data element from the software
    *     receive buffer. Zero value is returned if the software receive buffer
    *     is empty.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Next data element from the receive buffer.
    *
    * Global Variables:
    *  Look into SER_SpiInit for description.
    *
    *******************************************************************************/
    uint32 SER_SpiUartReadRxData(void)
    {
        uint32 rxData = 0u;

    #if (SER_INTERNAL_RX_SW_BUFFER_CONST)
        uint32 locTail;
    #endif /* (SER_INTERNAL_RX_SW_BUFFER_CONST) */

        #if (SER_CHECK_RX_SW_BUFFER)
        {
            if (SER_rxBufferHead != SER_rxBufferTail)
            {
                /* There is data in RX software buffer */

                /* Calculate index to read from */
                locTail = (SER_rxBufferTail + 1u);

                if (SER_INTERNAL_RX_BUFFER_SIZE == locTail)
                {
                    locTail = 0u;
                }

                /* Get data from RX software buffer */
                rxData = SER_GetWordFromRxBuffer(locTail);

                /* Change index in the buffer */
                SER_rxBufferTail = locTail;

                #if (SER_CHECK_UART_RTS_CONTROL_FLOW)
                {
                    /* Check if RX Not Empty is disabled in the interrupt */
                    if (0u == (SER_INTR_RX_MASK_REG & SER_INTR_RX_NOT_EMPTY))
                    {
                        /* Enable RX Not Empty interrupt source to continue
                        * receiving data into software buffer.
                        */
                        SER_INTR_RX_MASK_REG |= SER_INTR_RX_NOT_EMPTY;
                    }
                }
                #endif

            }
        }
        #else
        {
            /* Read data from RX FIFO */
            rxData = SER_RX_FIFO_RD_REG;
        }
        #endif

        return (rxData);
    }


    /*******************************************************************************
    * Function Name: SER_SpiUartGetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of received data elements in the receive buffer.
    *   - RX software buffer disabled: returns the number of used entries in
    *     RX FIFO.
    *   - RX software buffer enabled: returns the number of elements which were
    *     placed in the receive buffer. This does not include the hardware RX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Number of received data elements
    *
    *******************************************************************************/
    uint32 SER_SpiUartGetRxBufferSize(void)
    {
        uint32 size;
    #if (SER_INTERNAL_RX_SW_BUFFER_CONST)
        uint32 locHead;
    #endif /* (SER_INTERNAL_RX_SW_BUFFER_CONST) */

        #if (SER_CHECK_RX_SW_BUFFER)
        {
            locHead = SER_rxBufferHead;

            if(locHead >= SER_rxBufferTail)
            {
                size = (locHead - SER_rxBufferTail);
            }
            else
            {
                size = (locHead + (SER_INTERNAL_RX_BUFFER_SIZE - SER_rxBufferTail));
            }
        }
        #else
        {
            size = SER_GET_RX_FIFO_ENTRIES;
        }
        #endif

        return (size);
    }


    /*******************************************************************************
    * Function Name: SER_SpiUartClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the receive buffer and RX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_SpiUartClearRxBuffer(void)
    {
        #if (SER_CHECK_RX_SW_BUFFER)
        {
            /* Lock from component interruption */
            SER_DisableInt();

            /* Flush RX software buffer */
            SER_rxBufferHead = SER_rxBufferTail;
            SER_rxBufferOverflow = 0u;

            SER_CLEAR_RX_FIFO;
            SER_ClearRxInterruptSource(SER_INTR_RX_ALL);

            #if (SER_CHECK_UART_RTS_CONTROL_FLOW)
            {
                /* Enable RX Not Empty interrupt source to continue receiving
                * data into software buffer.
                */
                SER_INTR_RX_MASK_REG |= SER_INTR_RX_NOT_EMPTY;
            }
            #endif

            /* Release lock */
            SER_EnableInt();
        }
        #else
        {
            SER_CLEAR_RX_FIFO;
        }
        #endif
    }

#endif /* (SER_RX_DIRECTION) */


#if(SER_TX_DIRECTION)
    /*******************************************************************************
    * Function Name: SER_SpiUartWriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Places a data entry into the transmit buffer to be sent at the next available
    *  bus time.
    *  This function is blocking and waits until there is space available to put the
    *  requested data in the transmit buffer.
    *
    * Parameters:
    *  txDataByte: the data to be transmitted.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_SpiUartWriteTxData(uint32 txData)
    {
    #if (SER_INTERNAL_TX_SW_BUFFER_CONST)
        uint32 locHead;
    #endif /* (SER_INTERNAL_TX_SW_BUFFER_CONST) */

        #if (SER_CHECK_TX_SW_BUFFER)
        {
            /* Put data directly into the TX FIFO */
            if ((SER_txBufferHead == SER_txBufferTail) &&
                (SER_SPI_UART_FIFO_SIZE != SER_GET_TX_FIFO_ENTRIES))
            {
                /* TX software buffer is empty: put data directly in TX FIFO */
                SER_TX_FIFO_WR_REG = txData;
            }
            /* Put data into TX software buffer */
            else
            {
                /* Head index to put data */
                locHead = (SER_txBufferHead + 1u);

                /* Adjust TX software buffer index */
                if (SER_TX_BUFFER_SIZE == locHead)
                {
                    locHead = 0u;
                }

                /* Wait for space in TX software buffer */
                while (locHead == SER_txBufferTail)
                {
                }

                /* TX software buffer has at least one room */

                /* Clear old status of INTR_TX_NOT_FULL. It sets at the end of transfer when TX FIFO is empty. */
                SER_ClearTxInterruptSource(SER_INTR_TX_NOT_FULL);

                SER_PutWordInTxBuffer(locHead, txData);

                SER_txBufferHead = locHead;

                /* Check if TX Not Full is disabled in interrupt */
                if (0u == (SER_INTR_TX_MASK_REG & SER_INTR_TX_NOT_FULL))
                {
                    /* Enable TX Not Full interrupt source to transmit from software buffer */
                    SER_INTR_TX_MASK_REG |= (uint32) SER_INTR_TX_NOT_FULL;
                }
            }
        }
        #else
        {
            /* Wait until TX FIFO has space to put data element */
            while (SER_SPI_UART_FIFO_SIZE == SER_GET_TX_FIFO_ENTRIES)
            {
            }

            SER_TX_FIFO_WR_REG = txData;
        }
        #endif
    }


    /*******************************************************************************
    * Function Name: SER_SpiUartPutArray
    ********************************************************************************
    *
    * Summary:
    *  Places an array of data into the transmit buffer to be sent.
    *  This function is blocking and waits until there is a space available to put
    *  all the requested data in the transmit buffer. The array size can be greater
    *  than transmit buffer size.
    *
    * Parameters:
    *  wrBuf:  pointer to an array with data to be placed in transmit buffer.
    *  count:  number of data elements to be placed in the transmit buffer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_SpiUartPutArray(const uint8 wrBuf[], uint32 count)
    {
        uint32 i;

        for (i=0u; i < count; i++)
        {
            SER_SpiUartWriteTxData((uint32) wrBuf[i]);
        }
    }


    /*******************************************************************************
    * Function Name: SER_SpiUartGetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    * Returns the number of elements currently in the transmit buffer.
    *  - TX software buffer is disabled: returns the number of used entries in
    *    TX FIFO.
    *  - TX software buffer is enabled: returns the number of elements currently
    *    used in the transmit buffer. This number does not include used entries in
    *    the TX FIFO. The transmit buffer size is zero until the TX FIFO is
    *    not full.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Number of data elements ready to transmit.
    *
    *******************************************************************************/
    uint32 SER_SpiUartGetTxBufferSize(void)
    {
        uint32 size;
    #if (SER_INTERNAL_TX_SW_BUFFER_CONST)
        uint32 locTail;
    #endif /* (SER_INTERNAL_TX_SW_BUFFER_CONST) */

        #if (SER_CHECK_TX_SW_BUFFER)
        {
            /* Get current Tail index */
            locTail = SER_txBufferTail;

            if (SER_txBufferHead >= locTail)
            {
                size = (SER_txBufferHead - locTail);
            }
            else
            {
                size = (SER_txBufferHead + (SER_TX_BUFFER_SIZE - locTail));
            }
        }
        #else
        {
            size = SER_GET_TX_FIFO_ENTRIES;
        }
        #endif

        return (size);
    }


    /*******************************************************************************
    * Function Name: SER_SpiUartClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the transmit buffer and TX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_SpiUartClearTxBuffer(void)
    {
        #if (SER_CHECK_TX_SW_BUFFER)
        {
            /* Lock from component interruption */
            SER_DisableInt();

            /* Flush TX software buffer */
            SER_txBufferHead = SER_txBufferTail;

            SER_INTR_TX_MASK_REG &= (uint32) ~SER_INTR_TX_NOT_FULL;
            SER_CLEAR_TX_FIFO;
            SER_ClearTxInterruptSource(SER_INTR_TX_ALL);

            /* Release lock */
            SER_EnableInt();
        }
        #else
        {
            SER_CLEAR_TX_FIFO;
        }
        #endif
    }

#endif /* (SER_TX_DIRECTION) */


/*******************************************************************************
* Function Name: SER_SpiUartDisableIntRx
********************************************************************************
*
* Summary:
*  Disables the RX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  Returns the RX interrupt sources enabled before the function call.
*
*******************************************************************************/
uint32 SER_SpiUartDisableIntRx(void)
{
    uint32 intSource;

    intSource = SER_GetRxInterruptMode();

    SER_SetRxInterruptMode(SER_NO_INTR_SOURCES);

    return (intSource);
}


/*******************************************************************************
* Function Name: SER_SpiUartDisableIntTx
********************************************************************************
*
* Summary:
*  Disables TX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  Returns TX interrupt sources enabled before function call.
*
*******************************************************************************/
uint32 SER_SpiUartDisableIntTx(void)
{
    uint32 intSourceMask;

    intSourceMask = SER_GetTxInterruptMode();

    SER_SetTxInterruptMode(SER_NO_INTR_SOURCES);

    return (intSourceMask);
}


#if(SER_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SER_PutWordInRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Stores a byte/word into the RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  index:      index to store data byte/word in the RX buffer.
    *  rxDataByte: byte/word to store.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_PutWordInRxBuffer(uint32 idx, uint32 rxDataByte)
    {
        /* Put data in buffer */
        if (SER_ONE_BYTE_WIDTH == SER_rxDataBits)
        {
            SER_rxBuffer[idx] = ((uint8) rxDataByte);
        }
        else
        {
            SER_rxBuffer[(uint32)(idx << 1u)]      = LO8(LO16(rxDataByte));
            SER_rxBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(rxDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: SER_GetWordFromRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Reads byte/word from RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Returns byte/word read from RX buffer.
    *
    *******************************************************************************/
    uint32 SER_GetWordFromRxBuffer(uint32 idx)
    {
        uint32 value;

        if (SER_ONE_BYTE_WIDTH == SER_rxDataBits)
        {
            value = SER_rxBuffer[idx];
        }
        else
        {
            value  = (uint32) SER_rxBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32)SER_rxBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return (value);
    }


    /*******************************************************************************
    * Function Name: SER_PutWordInTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Stores byte/word into the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  idx:        index to store data byte/word in the TX buffer.
    *  txDataByte: byte/word to store.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_PutWordInTxBuffer(uint32 idx, uint32 txDataByte)
    {
        /* Put data in buffer */
        if (SER_ONE_BYTE_WIDTH == SER_txDataBits)
        {
            SER_txBuffer[idx] = ((uint8) txDataByte);
        }
        else
        {
            SER_txBuffer[(uint32)(idx << 1u)]      = LO8(LO16(txDataByte));
            SER_txBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(txDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: SER_GetWordFromTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Reads byte/word from the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  idx: index to get data byte/word from the TX buffer.
    *
    * Return:
    *  Returns byte/word read from the TX buffer.
    *
    *******************************************************************************/
    uint32 SER_GetWordFromTxBuffer(uint32 idx)
    {
        uint32 value;

        if (SER_ONE_BYTE_WIDTH == SER_txDataBits)
        {
            value = (uint32) SER_txBuffer[idx];
        }
        else
        {
            value  = (uint32) SER_txBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32) SER_txBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return (value);
    }

#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */


/* [] END OF FILE */
